/*
 * SonarQube Java
 * Copyright (C) 2012-2020 SonarSource SA
 * mailto:info AT sonarsource DOT com
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
package org.sonar.java.model.expression;

import java.util.Optional;
import org.sonar.java.model.AbstractTypedTree;
import org.sonar.java.model.ExpressionUtils;
import org.sonar.plugins.java.api.tree.ExpressionTree;

/**
 * Parent class for all ExpressionTrees, which allows to express them as their constant value
 */
public abstract class AssessableExpressionTree extends AbstractTypedTree implements ExpressionTree {

  @Override
  public Optional<Object> asConstant() {
    return Optional.ofNullable(ExpressionUtils.resolveAsConstant(this));
  }

  @Override
  public <T> Optional<T> asConstant(Class<T> type) {
    return asConstant().filter(type::isInstance).map(type::cast);
  }

}
